package speiger.src.coreengine.rendering.newGui.layout.constraints;

import speiger.src.coreengine.rendering.gui.helper.box.IGuiBox;
import speiger.src.coreengine.rendering.newGui.components.base.GuiComponent;

public interface IConstraint {
	public void apply(GuiComponent owner, GuiComponent child, Target target);
	
	public static interface ISimpleConstraint extends IConstraint {
		@Override
		default void apply(GuiComponent owner, GuiComponent parent, Target target) { apply(owner.getBox(), parent.getBox(), target); }
		public void apply(IGuiBox owner, IGuiBox parent, Target target);
	}
	
	public static enum Target {
		X,
		Y,
		WIDTH,
		HEIGHT;
		
		static final Target[] BY_INDEX = values();
		
		public static Target by(int index) { return BY_INDEX[index & 3]; }
		public static Target pos(boolean x) { return x ? X : Y; }
		public static Target bounds(boolean x) { return x ? WIDTH : HEIGHT; }
		public boolean isPosition() { return this == X || this == Y; }
		public Target asArea() {
			return switch(this) {
				case X -> WIDTH;
				case Y -> HEIGHT;
				default -> this;
			};
		}
		
		public float get(GuiComponent comp) { return get(comp.getBox()); }
		public void set(GuiComponent comp, float value) { set(comp.getBox(), value); }
		
		public float get(IGuiBox box) {
			return switch(this) {
				case X -> box.getMinX();
				case Y -> box.getMinY();
				case WIDTH -> box.getBaseWidth();
				case HEIGHT -> box.getBaseHeight();
			};
		}
		
		public void set(IGuiBox box, float value) {
			switch(this) {
				case X -> box.setX(value);
				case Y -> box.setY(value);
				case WIDTH -> box.setWidth(value);
				case HEIGHT -> box.setHeight(value);
			}
		}
	}
}
